import { isAuthorisedAsync } from "../util";
import { getRepository } from "typeorm";
import { Quote } from "@db/entity/Quote";
import { CommandSet, Command } from "src/model/command";
import { Message } from "discord.js";

const quotePattern = /add quote by "([^"]+)"\s*(.*)/i;

@CommandSet
export class QuoteCommand {

    minify(str: string, maxLength: number) {
        let result = str.replace("\n", "");
        if (result.length > maxLength)
            result = `${result.substring(0, maxLength - 3)}...`;
        return result;
    }

    @Command({ pattern: "add quote", auth: true, documentation: {description: "Adds a quote", example: "add quote by \"<NAME>\" <NEWLINE> <QUOTE>"} })
    async addQuote(msg: Message, c: string) {
        if (!isAuthorisedAsync(msg.member))
            return;

        let result = quotePattern.exec(c);

        if (result == null)
            return;

        let author = result[1].trim();
        let message = result[2].trim();

        let repo = getRepository(Quote);

        let newQuote = await repo.save(repo.create({
            author: author,
            message: message
        }));

        msg.channel.send(`${msg.author.toString()} Added quote (ID: ${newQuote.id})!`);
    }

    @Command({ pattern: "random quote", documentation: {description: "Shows a random quote by someone special...", example: "random quote"} })
    async postRandomQuote(msg: Message) {
        let repo = getRepository(Quote);

        let quotes = await repo.query(`  select *
                                                from quote
                                                order by random()
                                                limit 1`) as Quote[];

        if (quotes.length == 0) {
            msg.channel.send("I have no quotes!");
            return;
        }

        let quote = quotes[0];
        msg.channel.send(`Quote #${quote.id}:\n*"${quote.message}"*\n- ${quote.author}`);
    }

    @Command({ pattern: "remove quote", auth: true, documentation: {description: "Removes quote. Use \"quotes\" to get the <quote_index>!", example: "remove quote <quote_index>"} })
    async removeQuote(msg: Message, c: string) {
        let quoteNum = c.substring("remove quote".length).trim();
        let val = parseInt(quoteNum);
        if (isNaN(val))
            return;

        let repo = getRepository(Quote);

        let res = await repo.delete({ id: val });
        if (res.affected == 0)
            return;

        msg.channel.send(`${msg.author.toString()} Removed quote #${val}!`);
    }

    @Command({ pattern: "quotes", documentation: {description: "Lists all known quotes.", example: "quotes"}, auth: true })
    async listQuotes(msg: Message) {
        if (!isAuthorisedAsync(msg.member)) {
            msg.channel.send(`${msg.author.toString()} To prevent spamming, only bot moderators can view all quotes!`);
            return;
        }

        let repo = getRepository(Quote);

        let quotes = await repo.find();

        if (quotes.length == 0) {
            msg.channel.send("I have no quotes!");
            return;
        }

        let quotesListing = quotes.reduce((p, c) => `${p}[${c.id}] "${this.minify(c.message, 10)}" by ${c.author}\n`, "\n");
        msg.channel.send(`${msg.author.toString()}I know the following quotes:\n\`\`\`${quotesListing}\`\`\``);
    }
}